use color_eyre::Result;
use subxt::{
    ext::sp_core::{Pair as PairT, sr25519::Pair},
    tx::{PairSigner, TxProgress}, 
    OnlineClient,
};
use tokio::sync::mpsc::UnboundedSender;

use crate::{action::Action, casper::CasperConfig, casper_network, types::ActionLevel};

pub async fn transfer_balance(
    action_tx: &UnboundedSender<Action>,
    api: &OnlineClient<CasperConfig>,
    sender: &[u8; 32],
    receiver: &[u8; 32],
    amount: &u128,
) -> Result<TxProgress<CasperConfig, OnlineClient<CasperConfig>>> {
    let receiver_id = subxt::utils::MultiAddress::Id(
        subxt::utils::AccountId32::from(*receiver)
    );

    let transfer_tx = casper_network::tx()
        .balances()
        .transfer_allow_death(receiver_id, *amount);

    let pair = Pair::from_seed(sender);
    let signer = PairSigner::<CasperConfig, Pair>::new(pair);

    match api
        .tx()
        .sign_and_submit_then_watch_default(&transfer_tx, &signer)
        .await {
            Ok(tx_progress) => {
                action_tx.send(Action::WalletLog(
                        format!("transfer transaction {} sent", tx_progress.extrinsic_hash()),
                        ActionLevel::Info))?;
                Ok(tx_progress)
            },
            Err(err) => {
                action_tx.send(Action::WalletLog(
                        format!("error during transfer: {err}"), ActionLevel::Error))?;
                Err(err.into())
            }
        }
}
